      program mcfm
************************************************************************
*                                                                      *
*  This is the main program for MCFM                                   *
*                                                                      *
*  The sequence of calls should always be:                             *
*   call mcfm_init          : basic variable initialization, print-out *
*   call mcfm_vegas(warmup) : warm-up the Vegas grid                   *
*   call mcfm_vegas(accum)  : accumulate results                       *
*   call mcfm_exit          : final processing and print-out           *
*                                                                      *
************************************************************************
      implicit none
      include 'constants.f'
      include 'efficiency.f'
      include 'maxwt.f'
      include 'eventbuffer.f'
      include 'gridinfo.f'
      integer itmx1,ncall1,itmx2,ncall2,itmxplots
      double precision integ,integ_err
      logical dryrun
      integer i,pflav,pbarflav,j
      double precision p(mxpart,4),wt
      common/iterat/itmx1,ncall1,itmx2,ncall2
      common/dryrun/dryrun
      double precision msq(fn:nf,fn:nf)

      !process dependent parameters
      integer nexternal
      parameter (nexternal=4)

      ! files
      character*140 filename,inputfile,outputfile

      ! rw_events arguments
      integer n,ic(7,mxpart),ievent
      double precision wgt,scale,aqcd,aqed,PP(0:4,mxpart)
      logical done
      character*140 buff

      double precision tree,nlo,wgt2,cxn,std
      integer nevents
      character*140 b
      logical overwrite_wgts,stop_at_9
      parameter (overwrite_wgts=.true.,stop_at_9=.false.)
      logical fixed_ps
      parameter (fixed_ps=.true.)
      double precision checkza(1:3)
      common/shaker/checkza
      double precision checkmcfm1,checkmcfm2,checkmcfm3
      double precision nloboosted,beta
* basic variable initialization, print-out
      call mcfm_init

* in initial phases, we don't want any unweighting to take place.
* this will be set true in the first call to getevent.
      unweight = .false.

      if (fixed_ps) then
      open(unit=67,file='pspoint')
      do i=1,2
        read(67,*) P(i,4),P(i,1),P(i,2),P(i,3)
        do j=1,4
          P(i,j)=-P(i,j)
        enddo
      enddo
      do i=4,4
        read(67,*) P(i,4),P(i,1),P(i,2),P(i,3)
      enddo
      do i=3,3        
        read(67,*) P(i,4),P(i,1),P(i,2),P(i,3)
      enddo
      do i=5,6        
        read(67,*) P(i,4),P(i,1),P(i,2),P(i,3)
      enddo
      call gg_WW(P,nlo)
      print*,'nlo =',nlo
      close(67)
      STOP
      endif


      filename='input'
      open(unit=10,file=filename,err=90)
      read(10,'(a140)',end=50)inputfile
      read(10,'(a140)')outputfile
      close(10) 
      filename=inputfile
      write(*,*) 'input  : ',inputfile
      open(unit=11,file=filename,err=90)
      filename=outputfile
      write(*,*) 'output : ',outputfile
      open(unit=12,file=filename,err=90)

      buff='  '
      do while (index(buff,"/init") .eq. 0)
      
        read(11,'(a140)',end=50)buff
        if (index(buff,"/init") .eq. 0) write(12,*)buff

      enddo
      write(12,*)buff

      cxn=0d0
      std=0d0
      nevents=0
      done=.false.
      do while (.not. done)

        call read_event(11,PP,wgt,n,ic,ievent,scale,aqcd,aqed,buff,done)
        
        if (.not. done) then

          nlo=0d0
          tree=0d0
          nevents=nevents+1
          call map(PP,P,ic,n,mxpart)
          
          call gg_ZZ(P,nlo)
C          read(buff,*)b,tree

C          if (abs(nlo/nloboosted-1d0) .gt. 1d-3) then
C            write(*,*) 'Warning : ',
C     &nlo/tree
C            do i=1,2
c              write(*,*) (PP(j,i), j=0,3)
c            enddo
C            do i=5,8
C              write(*,*) (PP(j,i), j=0,3)
c            enddo
C            write(*,*) 'mcfm = ',nlo
c            write(*,*) 'mg5  = ',tree
c          endif
C          if (abs(nlo/tree-1d0) .gt. 1d-3) then
C           if (nevents .lt. 10) then
C            print*,'WARNING : ',nlo/tree
C            print*,'          ',checkza(1),checkmcfm1,
C     &checkza(2),checkmcfm2,checkza(3),checkmcfm3
C            print*,'          '
C          else
C            STOP
C          endif
          if (stop_at_9) then
              write(*,'(a)') buff
              write(*,'(a,3e15.7)') '#',tree,nlo/tree
              if (nevents .ge. 9) STOP
          endif
        
          write(buff,'(a,2e15.7)') '#',nlo
          wgt2=wgt*nlo/tree
          cxn = cxn + wgt2
          std = std + wgt2**2
          if (.not. overwrite_wgts) wgt2=wgt 
          call write_event(12,PP,wgt2,n,ic,ievent,scale,aqcd,aqed,buff)
           if (mod(nevents,10000) .eq. 0) write(*,*) 'At event ',nevents
        endif

      enddo
      write(12,'(a)') '</LesHouchesEvents>'
C---- done
      close(12)
      close(11)

      write(*,'(a,2e15.7)') '#antoine',cxn,dsqrt(std-cxn**2/nevents)

      go to 99
50    write(*,*) 'eof'
      go to 99
90    write(*,*) 'i/o error : ',filename
      go to 99
99    continue 
      stop
      end
     

      subroutine map(P1,P2,ic,n,mxpart)
 
      implicit none

      integer i,j,k,l,n,mxpart,ic(7,mxpart)
      double precision P_temp(1:4),P1(0:4,mxpart),P2(mxpart,4)


      l=0
      do i=1,n
        if (ic(6,i) .eq. -1) then
          l=l+1
          do j=1,3
          P2(l,j)=-P1(j,i)
          enddo
          P2(l,4)=-P1(0,i)
        elseif (ic(6,i) .eq. 1) then
          l=l+1
          do j=1,3
          P2(l,j)=P1(j,i)
          enddo
          P2(l,4)=P1(0,i)
        endif
      enddo

C      do i=1,4
C        P_temp(i)=P2(3,i)
C        P2(3,i)=P2(4,i)
C        P2(4,i)=P_temp(i)
C      enddo

      return
      end
  
      subroutine read_event(lun,P,wgt,nexternal,ic,ievent,scale,aqcd,
     &aqed,buff2,done)
c********************************************************************
c     Reads one event from data file #lun
c     ic(*,1) = Particle ID
c     ic(*,2) = Mothup(1)
c     ic(*,3) = Mothup(2)
c     ic(*,4) = ICOLUP(1)
c     ic(*,5) = ICOLUP(2)
c     ic(*,6) = ISTUP   -1=initial state +1=final  +2=decayed
c     ic(*,7) = Helicity
c********************************************************************
      implicit none

      double precision pi
      parameter (pi = 3.1415926d0)
c
c     Arguments
c      
      integer lun
      integer nexternal, ic(7,*)
      logical done
      double precision P(0:4,*),wgt,aqcd,aqed,scale
      integer ievent
      character*140 buff2
c
c     Local
c
      integer i,j,k
      character*(256) buff
      double precision xdum1,xdum2
c
c     Global
c
      logical banner_open
      integer lun_ban
      common/to_banner/banner_open, lun_ban

      data lun_ban/37/
      data banner_open/.false./
c-----
c  Begin Code
c-----     
      buff2=' '
      done=.false.
      if (.not. banner_open) then
         open (unit=lun_ban, status='scratch')
         banner_open=.true.
      endif
 11   read(lun,'(a256)',end=99,err=99) buff
      do while(index(buff,"<event") .eq. 0)
         write(lun_ban,'(a)') buff
         read(lun,'(a256)',end=99,err=99) buff
      enddo
      read(lun,*,err=11, end=11) nexternal,ievent,wgt,scale,aqed,aqcd
      do i=1,nexternal
         read(lun,*,err=99,end=99) ic(1,i),ic(6,i),(ic(j,i),j=2,5),
     $     (p(j,i),j=1,3),p(0,i),p(4,i),xdum1,xdum2
         ic(7,i)=xdum2
      enddo
      do while(index(buff,"</event") .eq. 0)
         read(lun,'(a256)',end=99,err=99) buff
         if(buff(1:1).eq.'#') buff2=buff(1:140)
      enddo
c      gal(1) = sqrt(4d0*pi*aqed)
c      g      = sqrt(4d0*pi*aqcd)
      return
 99   done=.true.
      return
 55   format(i3,5e19.11)         
      end

      subroutine write_event(lun,P,wgt,nexternal,ic,ievent,scale,aqcd,
     &aqed,buff)
c********************************************************************
c     Writes one event from data file #lun according to LesHouches
c     ic(1,*) = Particle ID
c     ic(2.*) = Mothup(1)
c     ic(3,*) = Mothup(2)
c     ic(4,*) = ICOLUP(1)
c     ic(5,*) = ICOLUP(2)
c     ic(6,*) = ISTUP   -1=initial state +1=final  +2=decayed
c     ic(7,*) = Helicity
c********************************************************************
      implicit none
c
c     parameters
c
      double precision pi
      parameter (pi = 3.1415926d0)
c
c     Arguments
c      
      integer lun, ievent
      integer nexternal, ic(7,*)
      double precision P(0:4,*),wgt
      double precision aqcd, aqed, scale
      character*140 buff
c
c     Local
c
      integer i,j,k
c
c     Global
c

c-----
c  Begin Code
c-----     
c      aqed= gal(1)*gal(1)/4d0/pi
c      aqcd = g*g/4d0/pi

      write(lun,'(a)') '<event>'
      write(lun,'(i2,i4,4e15.7)') nexternal,ievent,wgt,scale,aqed,aqcd
      do i=1,nexternal
         write(lun,51) ic(1,i),ic(6,i),(ic(j,i),j=2,5),
     $     (p(j,i),j=1,3),p(0,i),p(4,i),0.,real(ic(7,i))
      enddo
      if(buff(1:1).eq.'#') write(lun,'(a)') buff(1:len_trim(buff))
      write(lun,'(a)') '</event>'
      return
 51   format(i9,5i5,5e19.11,f3.0,f4.0)
      end

      subroutine write_comments(lun)
c********************************************************************
c     Outputs all of the banner comment lines back at the top of
c     the file lun.
c********************************************************************
      implicit none
c
c     Arguments
c
      integer lun
c
c     Local
c
      character*(80) buff
c
c     Global
c
      logical banner_open
      integer lun_ban
      common/to_banner/banner_open, lun_ban

c-----
c  Begin Code
c-----     
c      write(*,*) 'Writing comments'
      if (banner_open) then
         rewind(lun_ban)
         do while (.true.) 
            read(lun_ban,'(a79)',end=99,err=99) buff
            write(lun,'(a79)') buff
c            write(*,*) buff
         enddo
 99      close(lun_ban)
         banner_open = .false.
      endif
      end

