C      block data electroweak_input
C************************************************************************
C*     Calculational scheme for EW couplings                            *
C************************************************************************
Cc
Cc     ewscheme=-1  : Old MCFM default 
Cc                    input values = Gf,alpha(m_Z),m_W,m_Z
Cc                    output values = sin^2(theta_W),mtop
Cc
Cc     ewscheme=0   : Old MadEvent default (= AlpGen with iewopt=2)
Cc                    input values = sin^2(theta_W),alpha(m_Z),m_Z
Cc                    output values = m_W,Gf.
Cc
Cc     ewscheme=1   : New MCFM default, also Madevent default, "G_mu scheme"
Cc                    = LUSIFER and AlpGen (iewopt=3) defaults
Cc                    input values = G_F,m_Z,m_W
Cc                    output values = sin^2(theta_W),alpha(m_Z).
Cc
Cc     ewscheme=2   : input  values = G_F,sin^2(theta_W),alpha(m_Z)
Cc                    output values = m_W,m_Z.
Cc
Cc     ewscheme=3   : User choice. All parameters are left as they are
Cc                    input here. You have to know what you're doing.
Cc
C      implicit none
C      include 'ewinput.f'
C      data ewscheme  / +1                  /   ! Chooses EW scheme
C      data Gf_inp    / 1.16639d-5          /   ! G_F
C      data aemmz_inp / 7.7585538055706d-03 /   ! alpha_EM(m_Z)=1/128.89
C      data xw_inp    / 0.2312d0            /   ! sin^2(theta_W)
C      data wmass_inp / 80.398d0            /   ! W mass
C      data zmass_inp / 91.1876d0           /   ! Z mass
C      end
************************************************************************


      block data transversedefn
************************************************************************
*     Definition to use for computing transverse quantities            *
*         useEt=.false.    transverse momentum [previous default]      *
*         useEt=.true.     transverse energy                           *
************************************************************************
      implicit none
      include 'useet.f'
      data useEt/.false./
      end
************************************************************************


************************************************************************
*     Masses, widths and initial-state flavour information             *
************************************************************************
C      block data block_properties
C      implicit none
C      include 'masses.f'
C      include 'nflav.f'
C      include 'nores.f'
Cc--- if true, nores removes all of the gg contribution
C      data nores/.false./
Cc--- Masses: note that "mtausq" is typically used throughout the
Cc--- program to calculate couplings that depend on the mass, while
Cc--- "mtau" is the mass that appears in the rest of the matrix
Cc--- elements and phase space (and may be set to zero in the program,
Cc--- depending on the process number) 
C
C      data mtau,mtausq/1.777d0,3.157729d0/
Cc----   Note: after v5.6, the masses for top, bottom and charm quarks
Cc----         are set in the input file
C
Cc---  Widths: note that the top width is calculated in the program
Cc---  The W width of 2.1054 is derived using the measured BR of
Cc---    10.80 +/- 0.09 % (PDG) and the LO partial width calculation
Cc---    for Mw=80.398 GeV
C      data wwidth,zwidth/2.1054d0,2.4952d0/
C      data tauwidth/2.269d-12/
Cc--- Number of active flavours in the initial state: this parameter
Cc--- may be changed in the program for some processes
C      data nflav/5/
Cc--- Masses below here are currently unused      
C      data md,mu,ms/5d-3,5d-3,1d-1/
C      data mel,mmu/0.510997d-3,0.105658389d0/
C      end
************************************************************************


************************************************************************
*     CKM matrix entries                                               *
************************************************************************
      block data block_ckm
      implicit none
      double precision Vud,Vus,Vub,Vcd,Vcs,Vcb
      common/cabib/Vud,Vus,Vub,Vcd,Vcs,Vcb
      data  Vud  ,  Vus  ,  Vub  ,
     .      Vcd  ,  Vcs  ,  Vcb
     .   /1d0,0d0,0.000d0,
     .    0d0,1d0,0.000d0/
      end
************************************************************************


************************************************************************
*     Relevant for the H+b process only :                              *
*       mb_msbar: the value of the running b-mass, evaluated at the    * 
*                 pole mass. For negative values, calculated from mb   *
*       susycoup: the deviation of the Higgs coupling from the         *
*                 Standard Model value (S.M. = 1d0)                    *
************************************************************************
      block data block_bH
      implicit none
      include 'mb_msbar.f'
      include 'susycoup.f'
      data mb_msbar/4.25d0/
      data susycoup/1d0/
      end
************************************************************************


************************************************************************
*     Dim. Reg. parameter epsilon, used for checking the proper        *
*      operation of the NLO code in the program                        *
************************************************************************
      block data block_epinv
      implicit none
      include 'epinv.f'
      include 'epinv2.f'
      data epinv/ 1d3/
      data epinv2/1d3/
      end
************************************************************************

